@@ -3,11 +3,13 @@
 # This file is under the Apache License, Version 2.0.
 # See the file `LICENSE` for details.
 from aiven.client import envdefault, pretty
+from typing import Optional
 
 import aiven.client.client
 import argparse
 import csv as csvlib
 import errno
+import functools
 import json as jsonlib
 import logging
 import os
@@ -27,6 +29,11 @@ try:
 except ImportError:
     __version__ = "UNKNOWN"
 
+# cached_property only exists since python 3.8
+SKIP_EVALUATION_TYPES = (property, )
+if hasattr(functools, "cached_property"):
+    SKIP_EVALUATION_TYPES += (functools.cached_property, )
+
 ARG_LIST_PROP = "_arg_list"
 LOG_FORMAT = "%(levelname)s\t%(message)s"
 
@@ -116,12 +123,12 @@ class Config(dict):
 
 
 class CommandLineTool:  # pylint: disable=old-style-class
-    def __init__(self, name):
+    def __init__(self, name, parser: Optional[argparse.ArgumentParser] = None):
         self.log = logging.getLogger(name)
         self.config = None
         self._cats = {}
         self._extensions = []
-        self.parser = argparse.ArgumentParser(prog=name, formatter_class=CustomFormatter)
+        self.parser = parser or argparse.ArgumentParser(prog=name, formatter_class=CustomFormatter)
         self.parser.add_argument(
             "--config",
             help="config file location %(default)r",
@@ -133,7 +140,7 @@ class CommandLineTool:  # pylint: disable=old-style-class
 
     def add_cmd(self, func):
         """Add a parser for a single command method call"""
-        assert func.__doc__, func
+        assert func.__doc__, f"Missing docstring for {func.__qualname__}"
 
         # allow multi-level commands, separating each level with double underscores
         if "__" in func.__name__:
@@ -178,9 +185,9 @@ class CommandLineTool:  # pylint: disable=old-style-class
     def add_cmds(self, add_func):
         """Add every method tagged with @arg as a command"""
         for prop in dir(self):
-            # Skip @properties to avoid evaluating them
+            # Skip @property and @cached_property attributes to delay coercing their evaluation.
             classprop = getattr(self.__class__, prop, None)
-            if isinstance(classprop, property):
+            if isinstance(classprop, SKIP_EVALUATION_TYPES):
                 continue
             func = getattr(self, prop, None)
             if getattr(func, ARG_LIST_PROP, None) is not None:
@@ -295,8 +302,8 @@ class CommandLineTool:  # pylint: disable=old-style-class
         self.pre_run(func)
         return func()  # pylint: disable=not-callable
 
-    def main(self):
+    def main(self, args=None):
         # TODO: configurable log level
         logging.basicConfig(level=logging.INFO, format=LOG_FORMAT)
         logging.getLogger("requests").setLevel(logging.WARNING)
-        sys.exit(self.run())
+        sys.exit(self.run(args))
