@@ -3,7 +3,7 @@
 # This file is under the Apache License, Version 2.0.
 # See the file `LICENSE` for details.
 
-from .argx import arg
+from .argx import arg, UserError
 from functools import wraps
 
 import json as jsonlib
@@ -11,6 +11,9 @@ import os
 
 
 def get_json_config(path_or_string):
+    # If parameter is empty, return an empty dict
+    if not path_or_string:
+        return {}
     if path_or_string.startswith("@"):
         filepath = path_or_string[1:]
         with open(filepath, "r") as config_file:
@@ -40,6 +43,36 @@ def json_path_or_string(param_name):
     return wrapper
 
 
+def user_config_json():
+    """User config that accepts arbitrary JSON"""
+
+    def wrapper(fun):
+        arg(
+            "--user-config-json",
+            default=None,
+            dest="user_config_json",
+            help="JSON string or path (preceded by '@') to a JSON configuration file",
+        )(fun)
+
+        @wraps(fun)
+        def wrapped(self):
+            if ("user_config" in self.args and (self.args.user_config_json and self.args.user_config)):
+                raise UserError("-c (user config) and --user-config-json parameters can not be used at the same time")
+            try:
+                setattr(
+                    self.args,
+                    "user_config_json",
+                    get_json_config(self.args.user_config_json),
+                )
+            except jsonlib.decoder.JSONDecodeError as err:
+                raise UserError from err
+            return fun(self)
+
+        return wrapped
+
+    return wrapper
+
+
 arg.account_id = arg("account_id", help="Account identifier")
 arg.authentication_id = arg("authentication_id", help="Account authentication method identifier")
 arg.billing_address = arg("--billing-address", help="Physical billing address for invoices")
@@ -51,6 +84,7 @@ arg.billing_extra_text = arg(
 arg.billing_group = arg("id", help="Billing group ID")
 arg.card_id = arg("--card-id", help="Card ID")
 arg.cloud = arg("--cloud", help="Cloud to use (see 'cloud list' command)")
+arg.cloud_mandatory = arg("--cloud", help="Cloud to use (see 'cloud list' command)", required=True)
 arg.config_cmdline = arg(
     "-c",
     dest="config_cmdline",
@@ -106,7 +140,9 @@ arg.untag = arg(
     "--untag", dest="topic_option_untag", metavar="KEY", action="append", help="Tag to delete from topic metadata"
 )
 arg.service_name = arg("service_name", help="Service name")
+arg.service_name_mandatory = arg("service_name", help="Service name", required=True)
 arg.service_type = arg("-t", "--service-type", help="Type of service (see 'service list-types')")
+arg.static_ip_id = arg("static_ip_id", help="Static IP address ID")
 arg.ns_name = arg("ns_name", help="Namespace name")
 arg.ns_type = arg("--ns-type", help="Namespace type ('aggregated' or 'unaggregated')", required=True)
 arg.ns_retention_mandatory = arg(
@@ -133,6 +169,7 @@ arg.user_config = arg(
     default=[],
     help="Apply a configuration setting. See 'avn service types -v' for available values.",
 )
+arg.user_config_json = user_config_json
 arg.user_id = arg("--user-id", help="User identifier", required=True)
 arg.user_option_remove = arg(
     "--remove-option",
