@@ -2,6 +2,7 @@
 #
 # This file is under the Apache License, Version 2.0.
 # See the file `LICENSE` for details.
+from .session import get_requests_session
 from urllib.parse import quote
 
 import json
@@ -32,17 +33,12 @@ UNDEFINED = object()
 class AivenClientBase:  # pylint: disable=old-style-class
     """Aiven Client with low-level HTTP operations"""
 
-    def __init__(self, base_url, show_http=False):
+    def __init__(self, base_url, show_http=False, request_timeout=None):
         self.log = logging.getLogger("AivenClient")
         self.auth_token = None
         self.base_url = base_url
         self.log.debug("using %r", self.base_url)
-        self.session = requests.Session()
-        self.session.verify = True
-        self.session.headers = {
-            "content-type": "application/json",
-            "user-agent": "aiven-client/" + __version__,
-        }
+        self.session = get_requests_session(timeout=request_timeout)
         self.http_log = logging.getLogger("aiven_http")
         self.init_http_logging(show_http)
         self.api_prefix = "/v1"
@@ -392,14 +388,11 @@ class AivenClient(AivenClientBase):
             self.build_path("project", project, "integration", integration_id),
         )
 
-    def create_service_task(self, project, service, operation, target_version):
+    def create_service_task(self, project, service, body):
         return self.verify(
             self.post,
             self.build_path("project", project, "service", service, "task"),
-            body={
-                "task_type": operation,
-                "target_version": target_version,
-            },
+            body=body,
         )
 
     def get_service_task(self, project, service, task_id):
@@ -567,21 +560,23 @@ class AivenClient(AivenClientBase):
         retention_bytes,
         retention_hours,
         cleanup_policy,
-        tags,
+        tags=None,
     ):
+        body = {
+            "cleanup_policy": cleanup_policy,
+            "min_insync_replicas": min_insync_replicas,
+            "topic_name": topic,
+            "partitions": partitions,
+            "replication": replication,
+            "retention_bytes": retention_bytes,
+            "retention_hours": retention_hours,
+        }
+        if tags is not None:
+            body.update({"tags": tags})
         return self.verify(
             self.post,
             self.build_path("project", project, "service", service, "topic"),
-            body={
-                "cleanup_policy": cleanup_policy,
-                "min_insync_replicas": min_insync_replicas,
-                "topic_name": topic,
-                "partitions": partitions,
-                "replication": replication,
-                "retention_bytes": retention_bytes,
-                "retention_hours": retention_hours,
-                "tags": tags,
-            },
+            body=body,
         )
 
     def update_service_topic(
@@ -1159,6 +1154,7 @@ class AivenClient(AivenClientBase):
         project_vpc_id=UNDEFINED,
         service_integrations=None,
         termination_protection=False,
+        static_ips=(),
     ):
         user_config = user_config or {}
         body = {
@@ -1169,6 +1165,7 @@ class AivenClient(AivenClientBase):
             "service_type": service_type,
             "user_config": user_config,
             "termination_protection": termination_protection,
+            "static_ips": static_ips,
         }
         if project_vpc_id is not UNDEFINED:
             body["project_vpc_id"] = project_vpc_id
@@ -1215,8 +1212,8 @@ class AivenClient(AivenClientBase):
         path = self.build_path("project", project, "service", service, "credentials", "reset")
         return self.verify(self.put, path, result_key="service")
 
-    def _privatelink_path(self, project, service, cloud_provider):
-        return self.build_path("project", project, "service", service, "privatelink", cloud_provider)
+    def _privatelink_path(self, project, service, cloud_provider, *rest):
+        return self.build_path("project", project, "service", service, "privatelink", cloud_provider, *rest)
 
     def create_service_privatelink_aws(self, project, service, principals):
         path = self._privatelink_path(project, service, "aws")
@@ -1238,10 +1235,65 @@ class AivenClient(AivenClientBase):
         path = self._privatelink_path(project, service, "aws") + "/connections"
         return self.verify(self.get, path, result_key="connections")
 
+    def create_service_privatelink_azure(self, project, service, user_subscription_ids):
+        path = self._privatelink_path(project, service, "azure")
+        return self.verify(self.post, path, body={"user_subscription_ids": user_subscription_ids})
+
+    def refresh_service_privatelink_azure(self, project, service):
+        path = self._privatelink_path(project, service, "azure", "refresh")
+        return self.verify(self.post, path)
+
+    def update_service_privatelink_azure(self, project, service, user_subscription_ids):
+        path = self._privatelink_path(project, service, "azure")
+        return self.verify(self.put, path, body={"user_subscription_ids": user_subscription_ids})
+
+    def update_service_privatelink_connection_azure(self, project, service, privatelink_connection_id, user_ip_address):
+        path = self._privatelink_path(project, service, "azure", "connections", privatelink_connection_id)
+        return self.verify(self.put, path, body={"user_ip_address": user_ip_address})
+
+    def approve_service_privatelink_connection_azure(self, project, service, privatelink_connection_id):
+        path = self._privatelink_path(project, service, "azure", "connections", privatelink_connection_id, "approve")
+        return self.verify(self.post, path)
+
+    def delete_service_privatelink_azure(self, project, service):
+        path = self._privatelink_path(project, service, "azure")
+        return self.verify(self.delete, path)
+
+    def get_service_privatelink_azure(self, project, service):
+        path = self._privatelink_path(project, service, "azure")
+        return self.verify(self.get, path)
+
+    def list_service_privatelink_azure_connections(self, project, service):
+        path = self._privatelink_path(project, service, "azure") + "/connections"
+        return self.verify(self.get, path, result_key="connections")
+
     def list_privatelink_cloud_availability(self, project):
         path = self.build_path("project", project, "privatelink-availability")
         return self.verify(self.get, path, result_key="privatelink_availability")
 
+    def _static_ip_address_path(self, project, *parts):
+        return self.build_path("project", project, "static-ips", *parts)
+
+    def list_static_ip_addresses(self, project):
+        path = self._static_ip_address_path(project)
+        return self.verify(self.get, path, result_key="static_ips")
+
+    def create_static_ip_address(self, project, cloud_name):
+        path = self._static_ip_address_path(project)
+        return self.verify(self.post, path, body={"cloud_name": cloud_name})
+
+    def associate_static_ip_address(self, project, static_ip_id, service_name):
+        path = self._static_ip_address_path(project, static_ip_id, "association")
+        return self.verify(self.post, path, body={"service_name": service_name})
+
+    def dissociate_static_ip_address(self, project, static_ip_id):
+        path = self._static_ip_address_path(project, static_ip_id, "association")
+        return self.verify(self.delete, path)
+
+    def delete_static_ip_address(self, project, static_ip_id):
+        path = self._static_ip_address_path(project, static_ip_id)
+        return self.verify(self.delete, path)
+
     def delete_service(self, project, service):
         return self.verify(self.delete, self.build_path("project", project, "service", service))
 
@@ -1373,6 +1425,7 @@ class AivenClient(AivenClientBase):
         vat_id=None,
         billing_emails=None,
         tech_emails=None,
+        use_source_project_billing_group=None,
     ):
         body = {
             "card_id": card_id,
@@ -1399,6 +1452,8 @@ class AivenClient(AivenClientBase):
             body["billing_emails"] = [{"email": email} for email in billing_emails]
         if tech_emails is not None:
             body["tech_emails"] = [{"email": email} for email in tech_emails]
+        if use_source_project_billing_group is not None:
+            body["use_source_project_billing_group"] = use_source_project_billing_group
 
         return self.verify(self.post, "/project", body=body, result_key="project")
 
